@@ -5,7 +5,7 @@
 
     The standard domain.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,8 +13,9 @@ import re
 import unicodedata
 
 from six import iteritems
+
 from docutils import nodes
-from docutils.parsers.rst import directives
+from docutils.parsers.rst import Directive, directives
 from docutils.statemachine import ViewList
 
 from sphinx import addnodes
@@ -22,25 +23,35 @@ from sphinx.roles import XRefRole
 from sphinx.locale import l_, _
 from sphinx.domains import Domain, ObjType
 from sphinx.directives import ObjectDescription
-from sphinx.util import ws_re
+from sphinx.util import ws_re, logging, docname_join
 from sphinx.util.nodes import clean_astext, make_refnode
-from sphinx.util.compat import Directive
+
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, Iterator, List, Tuple, Type, Union  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.util.typing import RoleFunction  # NOQA
+
+logger = logging.getLogger(__name__)
 
 
 # RE for option descriptions
-option_desc_re = re.compile(r'((?:/|--|-|\+)?[-+\.?@#_a-zA-Z0-9]+)(=?\s*.*)')
+option_desc_re = re.compile(r'((?:/|--|-|\+)?[^\s=]+)(=?\s*.*)')
 # RE for grammar tokens
-token_re = re.compile('`(\w+)`', re.U)
+token_re = re.compile(r'`(\w+)`', re.U)
 
 
 class GenericObject(ObjectDescription):
     """
     A generic x-ref directive registered with Sphinx.add_object_type().
     """
-    indextemplate = ''
-    parse_node = None
+    indextemplate = ''  # type: unicode
+    parse_node = None  # type: Callable[[GenericObject, BuildEnvironment, unicode, addnodes.desc_signature], unicode]  # NOQA
 
     def handle_signature(self, sig, signode):
+        # type: (unicode, addnodes.desc_signature) -> unicode
         if self.parse_node:
             name = self.parse_node(self.env, sig, signode)
         else:
@@ -51,6 +62,7 @@ class GenericObject(ObjectDescription):
         return name
 
     def add_target_and_index(self, name, sig, signode):
+        # type: (unicode, unicode, addnodes.desc_signature) -> None
         targetname = '%s-%s' % (self.objtype, name)
         signode['ids'].append(targetname)
         self.state.document.note_explicit_target(signode)
@@ -78,6 +90,7 @@ class EnvVarXRefRole(XRefRole):
     """
 
     def result_nodes(self, document, env, node, is_ref):
+        # type: (nodes.Node, BuildEnvironment, nodes.Node, bool) -> Tuple[List[nodes.Node], List[nodes.Node]]  # NOQA
         if not is_ref:
             return [node], []
         varname = node['reftarget']
@@ -102,9 +115,10 @@ class Target(Directive):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
     def run(self):
+        # type: () -> List[nodes.Node]
         env = self.state.document.settings.env
         # normalize whitespace in fullname like XRefRole does
         fullname = ws_re.sub(' ', self.arguments[0].strip())
@@ -136,19 +150,18 @@ class Cmdoption(ObjectDescription):
     """
 
     def handle_signature(self, sig, signode):
+        # type: (unicode, addnodes.desc_signature) -> unicode
         """Transform an option description into RST nodes."""
         count = 0
         firstname = ''
         for potential_option in sig.split(', '):
             potential_option = potential_option.strip()
-            m = option_desc_re.match(potential_option)
+            m = option_desc_re.match(potential_option)  # type: ignore
             if not m:
-                self.env.warn(
-                    self.env.docname,
-                    'Malformed option description %r, should '
-                    'look like "opt", "-opt args", "--opt args", '
-                    '"/opt args" or "+opt args"' % potential_option,
-                    self.lineno)
+                logger.warning('Malformed option description %r, should '
+                               'look like "opt", "-opt args", "--opt args", '
+                               '"/opt args" or "+opt args"', potential_option,
+                               location=(self.env.docname, self.lineno))
                 continue
             optname, args = m.groups()
             if count:
@@ -166,6 +179,7 @@ class Cmdoption(ObjectDescription):
         return firstname
 
     def add_target_and_index(self, firstname, sig, signode):
+        # type: (unicode, unicode, addnodes.desc_signature) -> None
         currprogram = self.env.ref_context.get('std:program')
         for optname in signode.get('allnames', []):
             targetname = optname.replace('/', '-')
@@ -197,9 +211,10 @@ class Program(Directive):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
     def run(self):
+        # type: () -> List[nodes.Node]
         env = self.state.document.settings.env
         program = ws_re.sub('-', self.arguments[0].strip())
         if program == 'None':
@@ -211,17 +226,20 @@ class Program(Directive):
 
 class OptionXRefRole(XRefRole):
     def process_link(self, env, refnode, has_explicit_title, title, target):
+        # type: (BuildEnvironment, nodes.Node, bool, unicode, unicode) -> Tuple[unicode, unicode]  # NOQA
         refnode['std:program'] = env.ref_context.get('std:program')
         return title, target
 
 
 def split_term_classifiers(line):
+    # type: (unicode) -> List[Union[unicode, None]]
     # split line into a term and classifiers. if no classifier, None is used..
     parts = re.split(' +: +', line) + [None]
     return parts
 
 
 def make_glossary_term(env, textnodes, index_key, source, lineno, new_id=None):
+    # type: (BuildEnvironment, List[nodes.Node], unicode, unicode, int, unicode) -> nodes.term
     # get a text-only representation of the term and register it
     # as a cross-reference target
     term = nodes.term('', '', *textnodes)
@@ -265,6 +283,7 @@ class Glossary(Directive):
     }
 
     def run(self):
+        # type: () -> List[nodes.Node]
         env = self.state.document.settings.env
         node = addnodes.glossary()
         node.document = self.state.document
@@ -275,7 +294,7 @@ class Glossary(Directive):
         # be* a definition list.
 
         # first, collect single entries
-        entries = []
+        entries = []  # type: List[Tuple[List[Tuple[unicode, unicode, int]], ViewList]]
         in_definition = True
         was_empty = True
         messages = []
@@ -329,7 +348,7 @@ class Glossary(Directive):
         for terms, definition in entries:
             termtexts = []
             termnodes = []
-            system_messages = []
+            system_messages = []  # type: List[unicode]
             for line, source, lineno in terms:
                 parts = split_term_classifiers(line)
                 # parse the term with inline markup
@@ -365,9 +384,10 @@ class Glossary(Directive):
 
 
 def token_xrefs(text):
+    # type: (unicode) -> List[nodes.Node]
     retnodes = []
     pos = 0
-    for m in token_re.finditer(text):
+    for m in token_re.finditer(text):  # type: ignore
         if m.start() > pos:
             txt = text[pos:m.start()]
             retnodes.append(nodes.Text(txt, txt))
@@ -390,13 +410,14 @@ class ProductionList(Directive):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {}
+    option_spec = {}  # type: Dict
 
     def run(self):
+        # type: () -> List[nodes.Node]
         env = self.state.document.settings.env
         objects = env.domaindata['std']['objects']
         node = addnodes.productionlist()
-        messages = []
+        messages = []  # type: List[nodes.Node]
         i = 0
 
         for rule in self.arguments[0].split('\n'):
@@ -437,7 +458,8 @@ class StandardDomain(Domain):
                          searchprio=-1),
         'envvar': ObjType(l_('environment variable'), 'envvar'),
         'cmdoption': ObjType(l_('program option'), 'option'),
-    }
+        'doc': ObjType(l_('document'), 'doc', searchprio=-1)
+    }  # type: Dict[unicode, ObjType]
 
     directives = {
         'program': Program,
@@ -446,7 +468,7 @@ class StandardDomain(Domain):
         'envvar': EnvVar,
         'glossary': Glossary,
         'productionlist': ProductionList,
-    }
+    }  # type: Dict[unicode, Type[Directive]]
     roles = {
         'option':  OptionXRefRole(warn_dangling=True),
         'envvar':  EnvVarXRefRole(),
@@ -463,18 +485,21 @@ class StandardDomain(Domain):
                             warn_dangling=True),
         # links to labels, without a different title
         'keyword': XRefRole(warn_dangling=True),
-    }
+        # links to documents
+        'doc':     XRefRole(warn_dangling=True, innernodeclass=nodes.inline),
+    }  # type: Dict[unicode, Union[RoleFunction, XRefRole]]
 
     initial_data = {
-        'progoptions': {},  # (program, name) -> docname, labelid
-        'objects': {},      # (type, name) -> docname, labelid
-        'citations': {},    # name -> docname, labelid
-        'labels': {         # labelname -> docname, labelid, sectionname
+        'progoptions': {},      # (program, name) -> docname, labelid
+        'objects': {},          # (type, name) -> docname, labelid
+        'citations': {},        # citation_name -> docname, labelid, lineno
+        'citation_refs': {},    # citation_name -> list of docnames
+        'labels': {             # labelname -> docname, labelid, sectionname
             'genindex': ('genindex', '', l_('Index')),
             'modindex': ('py-modindex', '', l_('Module Index')),
             'search':   ('search', '', l_('Search Page')),
         },
-        'anonlabels': {     # labelname -> docname, labelid
+        'anonlabels': {         # labelname -> docname, labelid
             'genindex': ('genindex', ''),
             'modindex': ('py-modindex', ''),
             'search':   ('search', ''),
@@ -487,6 +512,7 @@ class StandardDomain(Domain):
                 'the label must precede a section header)',
         'numref':  'undefined label: %(target)s',
         'keyword': 'unknown keyword: %(target)s',
+        'doc': 'unknown document: %(target)s',
         'option': 'unknown option: %(target)s',
         'citation': 'citation not found: %(target)s',
     }
@@ -495,18 +521,24 @@ class StandardDomain(Domain):
         nodes.figure: ('figure', None),
         nodes.table: ('table', None),
         nodes.container: ('code-block', None),
-    }
+    }  # type: Dict[nodes.Node, Tuple[unicode, Callable]]
 
     def clear_doc(self, docname):
+        # type: (unicode) -> None
         for key, (fn, _l) in list(self.data['progoptions'].items()):
             if fn == docname:
                 del self.data['progoptions'][key]
         for key, (fn, _l) in list(self.data['objects'].items()):
             if fn == docname:
                 del self.data['objects'][key]
-        for key, (fn, _l) in list(self.data['citations'].items()):
+        for key, (fn, _l, lineno) in list(self.data['citations'].items()):
             if fn == docname:
                 del self.data['citations'][key]
+        for key, docnames in list(self.data['citation_refs'].items()):
+            if docnames == [docname]:
+                del self.data['citation_refs'][key]
+            elif docname in docnames:
+                docnames.remove(docname)
         for key, (fn, _l, _l) in list(self.data['labels'].items()):
             if fn == docname:
                 del self.data['labels'][key]
@@ -515,6 +547,7 @@ class StandardDomain(Domain):
                 del self.data['anonlabels'][key]
 
     def merge_domaindata(self, docnames, otherdata):
+        # type: (List[unicode], Dict) -> None
         # XXX duplicates?
         for key, data in otherdata['progoptions'].items():
             if data[0] in docnames:
@@ -525,6 +558,11 @@ class StandardDomain(Domain):
         for key, data in otherdata['citations'].items():
             if data[0] in docnames:
                 self.data['citations'][key] = data
+        for key, data in otherdata['citation_refs'].items():
+            citation_refs = self.data['citation_refs'].setdefault(key, [])
+            for docname in data:
+                if docname in docnames:
+                    citation_refs.append(docname)
         for key, data in otherdata['labels'].items():
             if data[0] in docnames:
                 self.data['labels'][key] = data
@@ -533,19 +571,31 @@ class StandardDomain(Domain):
                 self.data['anonlabels'][key] = data
 
     def process_doc(self, env, docname, document):
+        # type: (BuildEnvironment, unicode, nodes.Node) -> None
         self.note_citations(env, docname, document)
+        self.note_citation_refs(env, docname, document)
         self.note_labels(env, docname, document)
 
     def note_citations(self, env, docname, document):
+        # type: (BuildEnvironment, unicode, nodes.Node) -> None
         for node in document.traverse(nodes.citation):
             label = node[0].astext()
             if label in self.data['citations']:
                 path = env.doc2path(self.data['citations'][label][0])
-                env.warn_node('duplicate citation %s, other instance in %s' %
-                              (label, path), node)
-            self.data['citations'][label] = (docname, node['ids'][0])
+                logger.warning('duplicate citation %s, other instance in %s', label, path,
+                               location=node, type='ref', subtype='citation')
+            self.data['citations'][label] = (docname, node['ids'][0], node.line)
+
+    def note_citation_refs(self, env, docname, document):
+        # type: (BuildEnvironment, unicode, nodes.Node) -> None
+        for node in document.traverse(addnodes.pending_xref):
+            if node['refdomain'] == 'std' and node['reftype'] == 'citation':
+                label = node['reftarget']
+                citation_refs = self.data['citation_refs'].setdefault(label, [])
+                citation_refs.append(docname)
 
     def note_labels(self, env, docname, document):
+        # type: (BuildEnvironment, unicode, nodes.Node) -> None
         labels, anonlabels = self.data['labels'], self.data['anonlabels']
         for name, explicit in iteritems(document.nametypes):
             if not explicit:
@@ -563,8 +613,9 @@ class StandardDomain(Domain):
                 # link and object descriptions
                 continue
             if name in labels:
-                env.warn_node('duplicate label %s, ' % name + 'other instance '
-                              'in ' + env.doc2path(labels[name][0]), node)
+                logger.warning('duplicate label %s, ' % name + 'other instance '
+                               'in ' + env.doc2path(labels[name][0]),
+                               location=node)
             anonlabels[name] = docname, labelid
             if node.tagname in ('section', 'rubric'):
                 sectname = clean_astext(node[0])  # node[0] == title node
@@ -583,8 +634,17 @@ class StandardDomain(Domain):
                 continue
             labels[name] = docname, labelid, sectname
 
+    def check_consistency(self):
+        # type: () -> None
+        for name, (docname, labelid, lineno) in iteritems(self.data['citations']):
+            if name not in self.data['citation_refs']:
+                logger.warning('Citation [%s] is not referenced.', name,
+                               type='ref', subtype='citation',
+                               location=(docname, lineno))
+
     def build_reference_node(self, fromdocname, builder, docname, labelid,
                              sectname, rolename, **options):
+        # type: (unicode, Builder, unicode, unicode, unicode, unicode, Any) -> nodes.Node
         nodeclass = options.pop('nodeclass', nodes.reference)
         newnode = nodeclass('', '', internal=True, **options)
         innernode = nodes.inline(sectname, sectname)
@@ -608,12 +668,15 @@ class StandardDomain(Domain):
         return newnode
 
     def resolve_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         if typ == 'ref':
             resolver = self._resolve_ref_xref
         elif typ == 'numref':
             resolver = self._resolve_numref_xref
         elif typ == 'keyword':
             resolver = self._resolve_keyword_xref
+        elif typ == 'doc':
+            resolver = self._resolve_doc_xref
         elif typ == 'option':
             resolver = self._resolve_option_xref
         elif typ == 'citation':
@@ -624,6 +687,7 @@ class StandardDomain(Domain):
         return resolver(env, fromdocname, builder, typ, target, node, contnode)
 
     def _resolve_ref_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         if node['refexplicit']:
             # reference to anonymous label; the reference uses
             # the supplied link caption
@@ -641,6 +705,7 @@ class StandardDomain(Domain):
                                          docname, labelid, sectname, 'ref')
 
     def _resolve_numref_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         if target in self.data['labels']:
             docname, labelid, figname = self.data['labels'].get(target, ('', '', ''))
         else:
@@ -651,7 +716,7 @@ class StandardDomain(Domain):
             return None
 
         if env.config.numfig is False:
-            env.warn_node('numfig is disabled. :numref: is ignored.', node)
+            logger.warning('numfig is disabled. :numref: is ignored.', location=node)
             return contnode
 
         target_node = env.get_doctree(docname).ids.get(labelid)
@@ -664,7 +729,8 @@ class StandardDomain(Domain):
             if fignumber is None:
                 return contnode
         except ValueError:
-            env.warn_node("no number is assigned for %s: %s" % (figtype, labelid), node)
+            logger.warning("no number is assigned for %s: %s", figtype, labelid,
+                           location=node)
             return contnode
 
         try:
@@ -674,7 +740,7 @@ class StandardDomain(Domain):
                 title = env.config.numfig_format.get(figtype, '')
 
             if figname is None and '{name}' in title:
-                env.warn_node('the link has no caption: %s' % title, node)
+                logger.warning('the link has no caption: %s', title, location=node)
                 return contnode
             else:
                 fignum = '.'.join(map(str, fignumber))
@@ -688,10 +754,10 @@ class StandardDomain(Domain):
                     # old style format (cf. "Fig.%s")
                     newtitle = title % fignum
         except KeyError as exc:
-            env.warn_node('invalid numfig_format: %s (%r)' % (title, exc), node)
+            logger.warning('invalid numfig_format: %s (%r)', title, exc, location=node)
             return contnode
         except TypeError:
-            env.warn_node('invalid numfig_format: %s' % title, node)
+            logger.warning('invalid numfig_format: %s', title, location=node)
             return contnode
 
         return self.build_reference_node(fromdocname, builder,
@@ -700,6 +766,7 @@ class StandardDomain(Domain):
                                          title=title)
 
     def _resolve_keyword_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         # keywords are oddballs: they are referenced by named labels
         docname, labelid, _ = self.data['labels'].get(target, ('', '', ''))
         if not docname:
@@ -707,7 +774,24 @@ class StandardDomain(Domain):
         return make_refnode(builder, fromdocname, docname,
                             labelid, contnode)
 
+    def _resolve_doc_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
+        # directly reference to document by source name; can be absolute or relative
+        refdoc = node.get('refdoc', fromdocname)
+        docname = docname_join(refdoc, node['reftarget'])
+        if docname not in env.all_docs:
+            return None
+        else:
+            if node['refexplicit']:
+                # reference with explicit title
+                caption = node.astext()
+            else:
+                caption = clean_astext(env.titles[docname])
+            innernode = nodes.inline(caption, caption, classes=['doc'])
+            return make_refnode(builder, fromdocname, docname, None, innernode)
+
     def _resolve_option_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         progname = node.get('std:program')
         target = target.strip()
         docname, labelid = self.data['progoptions'].get((progname, target), ('', ''))
@@ -729,9 +813,10 @@ class StandardDomain(Domain):
                             labelid, contnode)
 
     def _resolve_citation_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         from sphinx.environment import NoUri
 
-        docname, labelid = self.data['citations'].get(target, ('', ''))
+        docname, labelid, lineno = self.data['citations'].get(target, ('', '', 0))
         if not docname:
             if 'ids' in node:
                 # remove ids attribute that annotated at
@@ -751,6 +836,7 @@ class StandardDomain(Domain):
             raise
 
     def _resolve_obj_xref(self, env, fromdocname, builder, typ, target, node, contnode):
+        # type: (BuildEnvironment, unicode, Builder, unicode, unicode, nodes.Node, nodes.Node) -> nodes.Node  # NOQA
         objtypes = self.objtypes_for_role(typ) or []
         for objtype in objtypes:
             if (objtype, target) in self.data['objects']:
@@ -764,7 +850,8 @@ class StandardDomain(Domain):
                             labelid, contnode)
 
     def resolve_any_xref(self, env, fromdocname, builder, target, node, contnode):
-        results = []
+        # type: (BuildEnvironment, unicode, Builder, unicode, nodes.Node, nodes.Node) -> List[Tuple[unicode, nodes.Node]]  # NOQA
+        results = []  # type: List[Tuple[unicode, nodes.Node]]
         ltarget = target.lower()  # :ref: lowercases its target automatically
         for role in ('ref', 'option'):  # do not try "keyword"
             res = self.resolve_xref(env, fromdocname, builder, role,
@@ -785,11 +872,16 @@ class StandardDomain(Domain):
         return results
 
     def get_objects(self):
+        # type: () -> Iterator[Tuple[unicode, unicode, unicode, unicode, unicode, int]]
         # handle the special 'doc' reference here
         for doc in self.env.all_docs:
             yield (doc, clean_astext(self.env.titles[doc]), 'doc', doc, '', -1)
         for (prog, option), info in iteritems(self.data['progoptions']):
-            yield (option, option, 'cmdoption', info[0], info[1], 1)
+            if prog:
+                fullname = ".".join([prog, option])
+                yield (fullname, fullname, 'cmdoption', info[0], info[1], 1)
+            else:
+                yield (option, option, 'cmdoption', info[0], info[1], 1)
         for (type, name), info in iteritems(self.data['objects']):
             yield (name, name, type, info[0], info[1],
                    self.object_types[type].attrs['searchprio'])
@@ -802,13 +894,16 @@ class StandardDomain(Domain):
                 yield (name, name, 'label', info[0], info[1], -1)
 
     def get_type_name(self, type, primary=False):
+        # type: (ObjType, bool) -> unicode
         # never prepend "Default"
         return type.lname
 
     def is_enumerable_node(self, node):
+        # type: (nodes.Node) -> bool
         return node.__class__ in self.enumerable_nodes
 
     def get_numfig_title(self, node):
+        # type: (nodes.Node) -> unicode
         """Get the title of enumerable nodes to refer them using its title"""
         if self.is_enumerable_node(node):
             _, title_getter = self.enumerable_nodes.get(node.__class__, (None, None))
@@ -822,8 +917,10 @@ class StandardDomain(Domain):
         return None
 
     def get_figtype(self, node):
+        # type: (nodes.Node) -> unicode
         """Get figure type of nodes."""
         def has_child(node, cls):
+            # type: (nodes.Node, Type) -> bool
             return any(isinstance(child, cls) for child in node)
 
         if isinstance(node, nodes.section):
@@ -838,6 +935,7 @@ class StandardDomain(Domain):
             return figtype
 
     def get_fignumber(self, env, builder, figtype, docname, target_node):
+        # type: (BuildEnvironment, Builder, unicode, unicode, nodes.Node) -> Tuple[int, ...]
         if figtype == 'section':
             if builder.name == 'latex':
                 return tuple()
@@ -859,8 +957,24 @@ class StandardDomain(Domain):
                 # Maybe it is defined in orphaned document.
                 raise ValueError
 
+    def get_full_qualified_name(self, node):
+        # type: (nodes.Node) -> unicode
+        if node.get('reftype') == 'option':
+            progname = node.get('std:program')
+            command = ws_re.split(node.get('reftarget'))
+            if progname:
+                command.insert(0, progname)
+            option = command.pop()
+            if command:
+                return '.'.join(['-'.join(command), option])
+            else:
+                return None
+        else:
+            return None
+
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.add_domain(StandardDomain)
 
     return {
