@@ -5,7 +5,7 @@
 
     Simple requests package loader
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -24,14 +24,14 @@ try:
 except ImportError:
     # python-requests package in Debian jessie does not provide ``requests.packages.urllib3``.
     # So try to import the exceptions from urllib3 package.
-    from urllib3.exceptions import SSLError
+    from urllib3.exceptions import SSLError  # type: ignore
 
 try:
     from requests.packages.urllib3.exceptions import InsecureRequestWarning
 except ImportError:
     try:
         # for Debian-jessie
-        from urllib3.exceptions import InsecureRequestWarning
+        from urllib3.exceptions import InsecureRequestWarning  # type: ignore
     except ImportError:
         # for requests < 2.4.0
         InsecureRequestWarning = None
@@ -41,7 +41,7 @@ try:
 except ImportError:
     try:
         # for Debian-jessie
-        from urllib3.exceptions import InsecurePlatformWarning
+        from urllib3.exceptions import InsecurePlatformWarning  # type: ignore
     except ImportError:
         # for requests < 2.4.0
         InsecurePlatformWarning = None
@@ -75,11 +75,17 @@ else:
             'install requests-2.4.1+.'
         )
 
+if False:
+    # For type annotation
+    from typing import Any, Generator, Union  # NOQA
+    from sphinx.config import Config  # NOQA
+
 useragent_header = [('User-Agent',
                      'Mozilla/5.0 (X11; Linux x86_64; rv:25.0) Gecko/20100101 Firefox/25.0')]
 
 
 def is_ssl_error(exc):
+    # type: (Exception) -> bool
     """Check an exception is SSLError."""
     if isinstance(exc, SSLError):
         return True
@@ -93,6 +99,7 @@ def is_ssl_error(exc):
 
 @contextmanager
 def ignore_insecure_warning(**kwargs):
+    # type: (Any) -> Generator
     with warnings.catch_warnings():
         if not kwargs.get('verify') and InsecureRequestWarning:
             # ignore InsecureRequestWarning if verify=False
@@ -101,6 +108,7 @@ def ignore_insecure_warning(**kwargs):
 
 
 def _get_tls_cacert(url, config):
+    # type: (unicode, Config) -> Union[str, bool]
     """Get addiotinal CA cert for a specific URL.
 
     This also returns ``False`` if verification is disabled.
@@ -113,7 +121,7 @@ def _get_tls_cacert(url, config):
     if not certs:
         return True
     elif isinstance(certs, (string_types, tuple)):
-        return certs
+        return certs  # type: ignore
     else:
         hostname = urlsplit(url)[1]
         if '@' in hostname:
@@ -123,6 +131,7 @@ def _get_tls_cacert(url, config):
 
 
 def get(url, **kwargs):
+    # type: (unicode, Any) -> requests.Response
     """Sends a GET request like requests.get().
 
     This sets up User-Agent header and TLS verification automatically."""
@@ -136,6 +145,7 @@ def get(url, **kwargs):
 
 
 def head(url, **kwargs):
+    # type: (unicode, Any) -> requests.Response
     """Sends a HEAD request like requests.head().
 
     This sets up User-Agent header and TLS verification automatically."""
