@@ -5,7 +5,7 @@
 
     Test the autosummary extension.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,7 +13,7 @@ from six import iteritems, StringIO
 
 from sphinx.ext.autosummary import mangle_signature
 
-from util import etree_parse
+from sphinx.testing.util import etree_parse
 
 import pytest
 
@@ -106,16 +106,16 @@ def test_get_items_summary(app, status, warning):
                   'dummy_module.func')
     assert autosummary_items['func'] == func_attrs
 
+
 def str_content(elem):
     if elem.text is not None:
         return elem.text
     else:
         return ''.join(str_content(e) for e in elem)
 
+
 @pytest.mark.sphinx('xml', **default_kw)
 def test_escaping(app, status, warning):
-    from xml.etree import ElementTree
-
     app.builder.build_all()
 
     outdir = app.builder.outdir
@@ -126,3 +126,22 @@ def test_escaping(app, status, warning):
     title = etree_parse(docpage).find('section/title')
 
     assert str_content(title) == 'underscore_module_'
+
+
+@pytest.mark.sphinx('dummy', testroot='ext-autosummary')
+def test_autosummary_generate(app, status, warning):
+    app.builder.build_all()
+
+    module = (app.srcdir / 'generated' / 'autosummary_dummy_module.rst').text()
+    assert ('   .. autosummary::\n'
+            '   \n'
+            '      Foo\n'
+            '   \n' in module)
+
+    Foo = (app.srcdir / 'generated' / 'autosummary_dummy_module.Foo.rst').text()
+    assert '.. automethod:: __init__' in Foo
+    assert ('   .. autosummary::\n'
+            '   \n'
+            '      ~Foo.__init__\n'
+            '      ~Foo.bar\n'
+            '   \n' in Foo)
