@@ -5,7 +5,7 @@
 
     Test the Theme class.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -17,6 +17,7 @@ from sphinx.theming import ThemeError
 
 
 @pytest.mark.sphinx(
+    testroot='theming',
     confoverrides={'html_theme': 'ziptheme',
                    'html_theme_options.testopt': 'foo'})
 def test_theme_api(app, status, warning):
@@ -25,10 +26,11 @@ def test_theme_api(app, status, warning):
     # test Theme class API
     assert set(app.html_themes.keys()) == \
         set(['basic', 'default', 'scrolls', 'agogo', 'sphinxdoc', 'haiku',
-             'traditional', 'testtheme', 'ziptheme', 'epub', 'nature',
-             'pyramid', 'bizstyle', 'classic', 'nonav'])
-    assert app.html_themes['testtheme'] == app.srcdir / 'testtheme'
+             'traditional', 'epub', 'nature', 'pyramid', 'bizstyle', 'classic', 'nonav',
+             'test-theme', 'ziptheme', 'staticfiles', 'parent', 'child'])
+    assert app.html_themes['test-theme'] == app.srcdir / 'test_theme' / 'test-theme'
     assert app.html_themes['ziptheme'] == app.srcdir / 'ziptheme.zip'
+    assert app.html_themes['staticfiles'] == app.srcdir / 'test_theme' / 'staticfiles'
 
     # test Theme instance API
     theme = app.builder.theme
@@ -93,3 +95,18 @@ def test_double_inheriting_theme(app, status, warning):
 def test_nested_zipped_theme(app, status, warning):
     assert app.builder.theme.name == 'child'
     app.build()  # => not raises TemplateNotFound
+
+
+@pytest.mark.sphinx(testroot='theming',
+                    confoverrides={'html_theme': 'staticfiles'})
+def test_staticfiles(app, status, warning):
+    app.build()
+    assert (app.outdir / '_static' / 'staticimg.png').exists()
+    assert (app.outdir / '_static' / 'statictmpl.html').exists()
+    assert (app.outdir / '_static' / 'statictmpl.html').text() == (
+        '<!-- testing static templates -->\n'
+        '<html><project>Python</project></html>'
+    )
+
+    result = (app.outdir / 'index.html').text()
+    assert '<meta name="testopt" content="optdefault" />' in result
